﻿/* Author:  Jim Leichliter
 * Blog:  http://CaptivateDev.com
 * Modified Date:  March 29th, 2014 
 * Purpose:  Local Storage API for Captivate Variables (get/set)
 * Requirements:  jStorage, jQuery 1.4.2+, mPDF for certificate generation
 * Modified: December 30th, 2015 - added new variable to certificate. (email)
 * Modified: July 12th, 2016 - added new window opener to the PHP Counter file
 */
/* Begin Google Analytics Event Tracking */
(function(i, s, o, g, r, a, m) {
    i['GoogleAnalyticsObject'] = r;
    i[r] = i[r] || function() {
        (i[r].q = i[r].q || []).push(arguments)
    }, i[r].l = 1 * new Date();
    a = s.createElement(o),
        m = s.getElementsByTagName(o)[0];
    a.async = 1;
    a.src = g;
    m.parentNode.insertBefore(a, m)
})(window, document, 'script', '//www.google-analytics.com/analytics.js', 'ga');
/* Change the Traking ID (UA4556XXX-Y) and the domain descriptor (captivatedev.com) */
window.ga('create', 'UA-23742196-1', 'labour.gov.on.ca');
window.ga('send', 'pageview');
/* End Google Analytics Event Tracking */

if (typeof window.userstorage === 'undefined') {
    window.userstorage = {};
}

userstorage = {
    uniqueKey: 'default',

    storeCpVar: function(cpVariableName) {
        console.log('saveVar()  uniqueKey: ' + this.uniqueKey + '   cpVariableName: ' + cpVariableName);
        var cpVarValue = cputil.getVar(cpVariableName);

        //First get the object if one exists
        var objToSave = this.getStoredObject();

        //Add to the existing object
        objToSave[cpVariableName] = cpVarValue;

        //var objToSave = {cpVariableName: cpVariableName, value: cpVarValue};
        $.jStorage.set(this.uniqueKey, objToSave);

        //return the value of the Cp variable
        return cpVarValue;
    },

    forcePassFail: function(bolPassFail) {
        //First get the object if one exists
        var objToSave = this.getStoredObject();

        //Add to the existing object
        objToSave['cpQuizInfoPassFail'] = bolPassFail;
        $.jStorage.set(this.uniqueKey, objToSave);

        //return the value of the Cp variable
        return bolPassFail;
    },

    getCpVar: function(cpVariableName, bolSetCpVariable) {
        console.log('retrieveVar()  uniqueKey: ' + this.uniqueKey + '   cpVariableName: ' + cpVariableName);

        //bolSetCpValue is optional so set the default to true
        bolSetCpVariable = (typeof bolSetCpVariable === "undefined") ? true : bolSetCpVariable;

        var cpValue = '';

        //check if we have a key
        if ($.jStorage.get(this.uniqueKey)) {
            if (typeof $.jStorage.get(this.uniqueKey)[cpVariableName] !== 'undefined') {
                cpValue = $.jStorage.get(this.uniqueKey)[cpVariableName];
                console.log('cpValue: ' + cpValue);
                if (bolSetCpVariable) {
                    cputil.setVar(cpVariableName, cpValue);
                }
            }
        }
        return cpValue;
    },

    getUniqueKey: function() {
        return this.uniqueKey;
    },

    setUniqueKey: function(cpVariableAsKey, courseId) {
        var key = cputil.getVar(cpVariableAsKey) + courseId;
        key = key.toLowerCase();
        this.uniqueKey = key;
        return key;
    },

    getStoredObject: function() {
        if ($.jStorage.get(this.uniqueKey)) {
            return $.jStorage.get(this.uniqueKey);
        } else {
            return {};
        }
    },

    getStoredObjectAsString: function() {
        if ($.jStorage.get(this.uniqueKey)) {
            return JSON.stringify($.jStorage.get(this.uniqueKey));
        } else {
            return JSON.stringify({});
        }
    }

};

if (typeof window.cputil === 'undefined') {
    window.cputil = {};
}

if (typeof window.mol === 'undefined') {
    window.mol = {};
}

$(document).ready(function() {
    if (typeof window.coursestorage === 'undefined') {
        //clone a copy of userstorage using jquery
        window.coursestorage = {};
        coursestorage = $.extend(coursestorage, userstorage);
    }
});

window.onbeforeunload = function(e) {
    mol.storeResumeSlide();
    mol.storeQuizAttempts();
    mol.storeQuizPassFail();
};

mol = {

    logIn: function() {
        //Get Email Address and set unique key for storage
        var email = this.validateEmailAndSetUniqueKey();

        if (email !== '') {
            //Get password and compare
            var pw = userstorage.getCpVar('varPassword', false);
            var pwTypedIn = cputil.getVar('varPassword');

            if (pw == pwTypedIn) {
                //move to next slide
                cputil.setVar('cpCmndNextSlide', 1);

                //populate Captivate variables from local storage
                this.populateCaptivateVariablesFromStorage();

                //check if user can resume from where they left off
                this.resumeCourse();

            } else {
                alert('Your password does not match.  Please try again or create a new account');
            }
        }
    },

    forgotPassword: function() {
        //Get Email Address and set unique key for storage
        var email = this.validateEmailAndSetUniqueKey();

        if (email != '') {
            //Get Password
            var pw = userstorage.getCpVar('varPassword', false);
            //Show Password
            alert('Your Password Is: ' + pw);
        }
    },

    newLearner: function() {
        var passed = this.validateNewLearnerForm();

        if (!passed) return;

        //set Unique Key
        userstorage.setUniqueKey('varUsereMail', '');
        coursestorage.setUniqueKey('varUsereMail', cputil.getVar('varCourseId'));

        //Store data
        userstorage.storeCpVar('varPassword');
        userstorage.storeCpVar('varFirstName');
        userstorage.storeCpVar('varLastName');

        //move to next slide
        cputil.setVar('cpCmndNextSlide', 1);
    },

    validateEmailAndSetUniqueKey: function() {

        //Get Email Address and set unique key for storage
        var email = cputil.getVar('varUsereMail');

        if (email && email != '') {
            email = email.toLowerCase();
            //set Unique Key
            userstorage.setUniqueKey('varUsereMail', '');
            coursestorage.setUniqueKey('varUsereMail', cputil.getVar('varCourseId'));

            //check to see if email address is in storage as a key
            var ary = $.jStorage.index();
            var foundEmail = false;
            for (var item in ary) {
                if (ary[item].toString().toLowerCase() == email) {
                    foundEmail = true;
                    break;
                }
            }

            if (foundEmail == false) {
                alert('That e-Mail address was not found.  Please try a different e-Mail address or sign up as a new Learner');
                return '';
            }

        } else {
            alert('Please enter a valid e-Mail Address');
            email = '';
        }
        return email;
    },

    validateNewLearnerForm: function() {
        //Get Email Address and set unique key for storage
        var msg = '';
        var bolPassedValidation = true;

        var emailAddress = cputil.getVar('varUsereMail');
        if (varUsereMail == '' || varUsereMail == ' ') {
            msg += 'Please enter a valid e-Mail address.<br/>';
            bolPassedValidation = false;
        }

        var firstName = cputil.getVar('varFirstName');
        if (firstName == '' || firstName == ' ') {
            msg += 'Please enter your First Name.<br/>';
            bolPassedValidation = false;
        }

        var lastName = cputil.getVar('varLastName');
        if (lastName == '' || lastName == ' ') {
            msg += 'Please enter your Last Name.<br/>';
            bolPassedValidation = false;
        }

        var password = cputil.getVar('varPassword');
        if (password == '' || password == ' ') {
            msg += 'Please enter a valid Password.<br/>';
            bolPassedValidation = false;
        }

        if (bolPassedValidation == false) {
            alert(msg);
        }
        return bolPassedValidation;
    },

    getCertificate: function() {

        /* Change the buttonURL to your own URL */
        var buttonURL = 'http://www.labour.gov.on.ca/scripts/mpdf/certificatev16.php?id=';

        /*Use JSON string to pass in variables to the id URL Parameter*/
        var strJSON = '{"firstName":"$$varFirstName$$","lastName":"$$varLastName$$","course":"$$varCourseTitle$$","forcedownload":0,"language":"$$varLanguage$$","usermail":"$$varUsereMail$$"}';

        /*Default is to open in a new window.  Use '_self' to open in current window*/
        var target = '_blank';

        /*Replace the Captivate Variables in the JSON string with the real values*/
        strJSON = cputil.replaceCpVariablesInURL(strJSON);

        /*Base 64 Encode JSON*/
        buttonURL = buttonURL + cputil.base64_encode(strJSON);
		
		/*Set URL for PHP counter */
		var counterURL = 'http://www.labour.gov.on.ca/scripts/mpdf/writelog.php';
		
		/*Open the Counter URL with the specified target*/
        //var windowOpener2 = window.open(counterURL, target);

		

        /*Open the URL with the specified target*/
        var windowOpener = window.open(buttonURL, target);
    },

    storeQuizPassFail: function() {
        console.log('storeQuizPassFail()');

        //check to see if Learner already passed quiz:
        var quizPassFail = this.getQuizPassFail();

        if (quizPassFail) {
            return true;
        } else {
            /*Store quizPassed in Local Storage*/
            return coursestorage.storeCpVar('cpQuizInfoPassFail');
        }
    },

    getQuizPassFail: function() {
        /*if cpQuizInfoPassFail is not in storage as an index, it will return an empty string meaning the user has not completed the quiz yet.
          if the quiz has been taken previously, then it should return true or false*/
        return coursestorage.getCpVar('cpQuizInfoPassFail');
    },

    storeQuizAttempts: function() {
        return coursestorage.storeCpVar('cpQuizInfoAttempts');
    },

    getQuizAttempts: function() {
        return coursestorage.getCpVar('cpQuizInfoAttempts');
    },

    hasAttemptedQuiz: function() {
        var attempts = this.getQuizAttempts();
        if (attempts == '' || attempts == 0) {
            return false;
        } else {
            return true;
        }
    },

    storeResumeSlide: function() {
        console.log('storeResumeSlide()');
        /*Store cpInfoCurrentSlide in Local Storage*/
        coursestorage.storeCpVar('cpInfoCurrentSlide');
    },

    resumeCourse: function() {
        console.log('resumeCourse');

        /*Check to see if we have a value for cpInfoCurrentSlide*/
        var resumeSlide = coursestorage.getCpVar('cpInfoCurrentSlide');
        var passedQuiz = this.getQuizPassFail();

        if (resumeSlide != '' && resumeSlide > 2 && passedQuiz) {
            //User already passed quiz so take them where ever they want.
            var result = confirm('Would you like to resume from slide ' + resumeSlide + ', where you previously left off?');
            if (result) {
                cputil.setVar('cpCmndGotoSlideAndResume', resumeSlide - 1); //zero based index
            }
        }

        //User has taken quiz but did not pass or complete the quiz
        if (resumeSlide != '' && resumeSlide > 2 && this.hasAttemptedQuiz() && (passedQuiz == '' || passedQuiz == false)) {
            //User has taken but failed or did not complete the quiz
            alert('You have not passed the quiz.  You will need to start from the beginning in order to successfully complete the course.');
        }

        //User has not attempted quiz
        if (resumeSlide != '' && resumeSlide > 2 && this.hasAttemptedQuiz() == false && (passedQuiz == '' || passedQuiz == false)) {
            var result2 = confirm('Would you like to resume from slide ' + resumeSlide + ', where you previously left off?');
            if (result2) {
                cputil.setVar('cpCmndGotoSlideAndResume', resumeSlide - 1); //zero based index

            }
        }

    },

    populateCaptivateVariablesFromStorage: function() {
        userstorage.getCpVar('varFirstName');
        userstorage.getCpVar('varLastName');
        userstorage.getCpVar('varPassword');
    },

    skipQuizCheckAndGoToSlide: function(slideNumber) {
        //check to see if Learner can skip quiz:
        var quizPassFail = this.getQuizPassFail();

        //get the current slide and compare with slideNumber to determine if moving forward or backward
        var currentSlide = cputil.getVar('cpInfoCurrentSlide');

        if (quizPassFail) {
            cputil.setVar('cpCmndGotoSlideAndResume', slideNumber - 1); //zero based index
        } else {
            if (currentSlide < slideNumber) {
                //Moving forward
                //Move to next slide
                cputil.setVar('cpCmndNextSlide', 1);
            } else {
                // Move back one slide
                cputil.setVar('cpCmndPreviousSlide', 1);
            }

        }
    },

};

cputil = {
    setVar: function(cpUserVariableName, variableValue) {
        /* Check for HTML5 vs. SWF output */
        if (typeof window.cp === 'undefined') {
            /* We have SWF output, so Get the Captivate Object */
            var objCp = document.getElementById('Captivate');
            if (objCp && objCp.cpEISetValue) {
                /* Set the Captivate User variable with the JavaScript variable, variableValue */
                objCp.cpEISetValue('m_VarHandle.' + cpUserVariableName, variableValue);
            }
        } else {
            /* We have HTML5 output */
            /*cpCmnd variables only work off the window object so check for them*/
            if ((cpUserVariableName.indexOf('cpCmnd') > -1) || (cpUserVariableName.indexOf('cpLock') > -1)) {
                window[cpUserVariableName] = variableValue;
            } else {

                if (cp.vm && cp.vm.setVariableValue) {
                    cp.vm.setVariableValue(cpUserVariableName, variableValue);
                }
            }

        }
    },

    getVar: function(cpUserVariableName) {

        /* Check for HTML5 vs. SWF output */
        if (typeof window.cp === 'undefined') {
            var objCp = document.getElementById('Captivate');

            /* We have SWF output, so Get the Captivate Object */
            if (objCp && objCp.cpEIGetValue) {
                /* Get the Captivate User variable */
                return objCp.cpEIGetValue('m_VarHandle.' + cpUserVariableName);
            }
        } else {
            /* We have HTML5 output */
            /*If variable does not exist off of the window object, then use the variables manager*/
            if (typeof window[cpUserVariableName] === 'undefined') {
                if (cp.vm && cp.vm.getVariableValue) {
                    return cp.vm.getVariableValue(cpUserVariableName);
                } else {
                    return '';
                }
            } else {
                return window[cpUserVariableName];
            }
        }
    },

    base64_encode: function(data) {
        var b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
        var o1, o2, o3, h1, h2, h3, h4, bits, i = 0,
            ac = 0,
            enc = "",
            tmp_arr = [];
        if (!data) {
            return data;
        }
        do { // pack three octets into four hexets
            o1 = data.charCodeAt(i++);
            o2 = data.charCodeAt(i++);
            o3 = data.charCodeAt(i++);
            bits = o1 << 16 | o2 << 8 | o3;
            h1 = bits >> 18 & 0x3f;
            h2 = bits >> 12 & 0x3f;
            h3 = bits >> 6 & 0x3f;
            h4 = bits & 0x3f;
            // use hexets to index into b64, and append result to encoded string
            tmp_arr[ac++] = b64.charAt(h1) + b64.charAt(h2) + b64.charAt(h3) + b64.charAt(h4);
        } while (i < data.length);
        enc = tmp_arr.join('');
        var r = data.length % 3;
        return (r ? enc.slice(0, r - 3) : enc) + '==='.slice(r || 3);
    },

    replaceCpVariablesInURL: function(url) {
        var i, e, varValue, varName;

        while (url.indexOf("$$") != -1) {
            i = url.indexOf("$$", 0);
            e = url.indexOf("$$", i + 2);
            varName = url.substring(i + 2, e);

            /*Instead of true/false, show Pass/Fail for cpQuizInfoPassFail variable*/
            if (varName == 'cpQuizInfoPassFail') {
                if (varValue == false) {
                    varValue = 'Fail';
                } else {
                    varValue = 'Pass';
                }
            }

            /* encodeURIComponent escapes all characters except the following:   - _ . ! ~ * ' ( )             */
            varValue = encodeURIComponent(cputil.getVar(varName)).replace(/\-/g, "%2D").replace(/\_/g, "%5F").replace(/\./g, "%2E").replace(/\!/g, "%21").replace(/\~/g, "%7E").replace(/\*/g, "%2A").replace(/\'/g, "%27").replace(/\(/g, "%28").replace(/\)/g, "%29");

            /*varValue = cputil.getVar(varName);*/

            /*Join back the URL with all the new values*/
            url = url.split("$$" + url.substring(i + 2, e) + "$$").join(varValue);
        }
        return url;
    }
};